﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;

//autocomplete textbox info - http://net-informations.com/q/faq/autocomplete.html

namespace Capstone_Project___final
{
    public partial class FormAddCustomer : Form
    {
        string timePickup = "null";
        string timeDropoff = "null";

        //These collections hold the data that will be used for autocompleting the textboxes
        AutoCompleteStringCollection DataCollectionPlaces = new AutoCompleteStringCollection();
        AutoCompleteStringCollection DataCollectionFirst = new AutoCompleteStringCollection();
        AutoCompleteStringCollection DataCollectionLast = new AutoCompleteStringCollection();

        public FormAddCustomer(Customer[] custArray , string[] placesArray)
        {
            InitializeComponent();
            
            //put the time format in hh:mm format
            dtp_Time.CustomFormat = "hh:mm";
            
            //make the *'s red
            lab_StarReq.ForeColor = Color.Red;
            lab_Star1.ForeColor = Color.Red;
            lab_Star2.ForeColor = Color.Red;
            lab_Star3.ForeColor = Color.Red;
            lab_Star4.ForeColor = Color.Red;


            //sets custom source info for all of the customer data entry textboxes
            tb_FN.AutoCompleteMode = AutoCompleteMode.Suggest;
            tb_FN.AutoCompleteSource = AutoCompleteSource.CustomSource;

            tb_LN.AutoCompleteMode = AutoCompleteMode.Suggest;
            tb_LN.AutoCompleteSource = AutoCompleteSource.CustomSource;

            tb_DropoffAddr.AutoCompleteMode = AutoCompleteMode.Suggest;
            tb_DropoffAddr.AutoCompleteSource = AutoCompleteSource.CustomSource;

            tb_PickupAddr.AutoCompleteMode = AutoCompleteMode.Suggest;
            tb_PickupAddr.AutoCompleteSource = AutoCompleteSource.CustomSource;

            //adds name data to the first and last name completion collections
            int i = 0;  
            while (custArray[i].name != "")
            {
                string[] firstlast = custArray[i].name.Split(' '); 

                DataCollectionFirst.Add(firstlast[0]);
                DataCollectionLast.Add(firstlast[1]);
                i++;
            }

            //adds the places data to the place completion collections
            i = 0;
            while (placesArray[i] != "")
            {
                DataCollectionPlaces.Add(placesArray[i]);
                i++;
            }

            //assign the filled custom sources to their respective textboxes
            tb_FN.AutoCompleteCustomSource = DataCollectionFirst;
            tb_LN.AutoCompleteCustomSource = DataCollectionLast;
            tb_DropoffAddr.AutoCompleteCustomSource = DataCollectionPlaces; //both textboxes pull from same data
            tb_PickupAddr.AutoCompleteCustomSource = DataCollectionPlaces;
        }

        //get methods to access data in mainform
          public string firstname 
          {
            get { return tb_FN.Text; }
          }

          public string lastname
          {
              get { return tb_LN.Text; }
          }

          public string pickup
          {
              get { return tb_PickupAddr.Text; }
          }

          public string dropoff
          {
              get { return tb_DropoffAddr.Text; }
          }

          public string timePickupEntered
          {
              get { return timePickup; }
          }

          public string timeDropoffEntered
          {
              get { return timeDropoff; }
          }

        public bool hasWheelchair
        {
            get
            {
                if (rb_Yes.Checked)
                    return true;

                else
                    return false;
            }
        }

        //validates information when submit is clicked
        private void btn_Submit_Click(object sender, EventArgs e)
        {
            if (tb_FN.Text.Length == 0 || tb_LN.Text.Length == 0 || tb_PickupAddr.Text.Length == 0 || tb_DropoffAddr.Text.Length == 0)
                MessageBox.Show("Please enter all of the information!");

            else if (!System.Text.RegularExpressions.Regex.IsMatch(tb_FN.Text, "^[a-zA-Z]") || !System.Text.RegularExpressions.Regex.IsMatch(tb_LN.Text, "^[a-zA-Z]"))
            {
                MessageBox.Show("Please enter a valid name!");
            }

            else
            {
                //gets valid time information if entered
                DateTime dt = DateTime.Parse(dtp_Time.Text);

                if (!dtp_Time.Text.Contains("12:01"))   //if time field changed
                {
                    if (rb_PickupAM.Checked)
                        timePickup = dt.ToString("HH:mm") + " AM";
                    else
                        timePickup = dt.ToString("HH:mm") + " PM";
                }

                dt = DateTime.Parse(dtp_TimeDropoff.Text);

                if(!dtp_TimeDropoff.Text.Contains("12:01"))
                {
                    if (rb_DropoffAM.Checked)
                        timeDropoff = dt.ToString("HH:mm") + " AM";
                    else
                        timeDropoff = dt.ToString("HH:mm") + " PM";
                }

                this.DialogResult = DialogResult.OK;
                this.Close();
            }
        }

        //closes the form without taking back any data - cancel 
        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();            
        }

        //autocapitlizes first char of first name
        private void tb_FN_TextChanged(object sender, EventArgs e)  
        {
            if (tb_FN.TextLength != 0)
            {
                tb_FN.Text = char.ToUpper(tb_FN.Text[0]) + tb_FN.Text.Substring(1);
                tb_FN.SelectionStart = tb_FN.TextLength;
            }
        }

        //autocapitlizes first char of last name
        private void tb_LN_TextChanged(object sender, EventArgs e)  
        {
            if (tb_LN.TextLength != 0)
            {
                tb_LN.Text = char.ToUpper(tb_LN.Text[0]) + tb_LN.Text.Substring(1);
                tb_LN.SelectionStart = tb_LN.TextLength;
            }
        }

        //these two functions make it so the textboxes only allow alphabetic characters and the backspace for name
        private void tb_FN_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(char.IsLetter(e.KeyChar) || e.KeyChar == (char)Keys.Back);
        }

        private void tb_LN_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(char.IsLetter(e.KeyChar) || e.KeyChar == (char)Keys.Back);
        }


        //sets the time format to hh:mm
        private void dtp_Time_ValueChanged(object sender, EventArgs e)
        {
            dtp_Time.CustomFormat = "hh:mm";
        }

        //clears all fields when the clear button is clicked
        private void btn_Clear_Click(object sender, EventArgs e)
        {
            tb_FN.Text = "";
            tb_LN.Text = "";
            tb_PickupAddr.Text = "";
            tb_DropoffAddr.Text = "";
            rb_No.Checked = true;
            dtp_Time.Text = "12:01";
            dtp_TimeDropoff.Text = "12:01";
            rb_PickupAM.Checked = true;
            rb_DropoffAM.Checked = true;
        }

        //puts the current time in the pickup time box when this button is pressed
        private void btn_CTP_Click(object sender, EventArgs e)
        {
            DateTime curTime = DateTime.Now;
            dtp_Time.Text = curTime.ToShortTimeString();
            string ampm = curTime.ToString("tt");
            if (ampm == "am")
                rb_PickupAM.Checked = true;
            else
                rb_PickupPM.Checked = true;                
        }

        //puts the current time in the dropoff time box when this button is pressed
        private void btn_CTD_Click(object sender, EventArgs e)
        {
            DateTime curTime = DateTime.Now; 
            dtp_TimeDropoff.Text = curTime.ToShortTimeString();
            string ampm = curTime.ToString("tt");
            if (ampm == "am")
                rb_DropoffAM.Checked = true;
            else
                rb_DropoffPM.Checked = true;
        }

        //only allows the users to type letters, backspace, numbers, and separator keys in the dropoff address textbox
        private void tb_DropoffAddr_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(char.IsLetter(e.KeyChar) || e.KeyChar == (char)Keys.Back 
                     || char.IsDigit(e.KeyChar) || char.IsSeparator(e.KeyChar) || e.KeyChar == '.'
                         || e.KeyChar == ';' || e.KeyChar == ',');
        }

        //only allows the users to type letters, backspace, numbers, and separator keys in the pickup address textbox
        private void tb_PickupAddr_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(char.IsLetter(e.KeyChar) || e.KeyChar == (char)Keys.Back
                     || char.IsDigit(e.KeyChar) || char.IsSeparator(e.KeyChar) || e.KeyChar == '.'
                         || e.KeyChar == ';' || e.KeyChar == ',');
        }

        //sets the requested pickup time back to default 12:01
        private void btn_noTimePickup_Click(object sender, EventArgs e)
        {
            dtp_Time.Text = "12:01";
            rb_PickupAM.Checked = true;
        }

        //sets the requested dropoff time back to default 12:01
        private void btn_noTimeDropoff_Click(object sender, EventArgs e)
        {
            dtp_TimeDropoff.Text = "12:01";
            rb_DropoffAM.Checked = true;
        }
    }
}