﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Net;
using System.Xml.Linq;
using GMap.NET;
using GMap.NET.WindowsForms;
using GMap.NET.WindowsForms.Markers;
using GMap.NET.WindowsForms.ToolTips;
using GMap.NET.MapProviders;
using System.Threading;
using System.Runtime.CompilerServices;

//reorderable listbox - http://www.codeproject.com/Articles/36412/Drag-and-Drop-ListBox
//Gmap Control - http://www.codeproject.com/Articles/32643/GMap-NET-Great-Maps-for-Windows-Forms-and-Presenta
//examples - http://www.websofia.com/2013/02/gmap-net-tutorial-maps-markers-and-polygons/


namespace Capstone_Project___final
{ 
    public partial class FormMain : Form
    {
        //globals
        public Customer[] customerArray = new Customer[100];    //includes names from file and those added with new customers
        public string[] customerPlaces = new string[100];       //includes places from file and those added with new customers
        public string[] fileLineInfo = new string[100];         //contains names read from file on load only
        public string[] fileLinePlaces = new string[100];       //contains places read from file on load only
        public string[] customerInteractions = new string[100]; //will keep a list of all dropoffs/pickups done in a session
        int maxPassengers = 10;
        int numPassengers = 0;

        //overlays that keep track of all markers, routes
        GMarkerGoogle[] markerArray = new GMarkerGoogle[100];
        GMapOverlay pickupMarkers = new GMapOverlay("markers");
        GMapOverlay dropoffMarkers = new GMapOverlay("markersDrop");
        GMapOverlay mouseOverMarkers = new GMapOverlay("mouseOver");
        GMapOverlay taxiMarker = new GMapOverlay("taxi");
        GMapOverlay routesOverlay = new GMapOverlay("My route");
        GMapRoute r = new GMapRoute("My route");

        //adds custom "taxi" marker image   
        Image taxiImage = Image.FromFile("taxi.png");
       
        HelpWindow childHelp;
        bool testMode = false;
        int markerPairNum = 0;  //customer's pickup and dropoff pairnumber will be the same
        bool MBShown = false;  //var for makeing sure the error messages associated with pickup and dropoff (not enough space 
                               // or no one to drop off) will not show up more than once
        
        public FormMain()
        {            
            InitializeComponent();

            //puts me in control ob drawing the lb_Destinations listbox - allows for custom text colors
            lb_Destinations.DrawMode = DrawMode.OwnerDrawFixed;
            lb_Destinations.DrawItem += lb_Destinations_DrawItem;

            //initializes various arrays
            for (int i = 0; i < customerArray.Length; i++)
            {
                customerArray[i] = new Customer();
                customerArray[i].name = "";
                customerArray[i].dropoffAddr = "";
                customerArray[i].pickupAddr = "";
                customerPlaces[i] = "";
                fileLinePlaces[i] = "";
                fileLineInfo[i] = "";
                customerInteractions[i] = "";
            }

            //fill array with customer data stored in file
            StreamReader fileNames = new StreamReader("customerNames.txt", false);  //name; no/yes

            int count = 0;
            string fileLine = fileNames.ReadLine();
            while (fileLine != null && fileLine != "")
            {
                fileLineInfo[count] = fileLine;
                //full line of text from file
                string[] custInfo = fileLine.Split(';');            //now break up line to get customer info
                customerArray[count].name = custInfo[0].Trim();                //put info into array of customers

                if (custInfo[1] == " no\n" || custInfo[1] == " no")
                    customerArray[count].wheelchair = false;
                else
                    customerArray[count].wheelchair = true;

                count++;
                fileLine = fileNames.ReadLine();    //need 2 to avoid newline apparently..

                if(fileLine=="")
                    fileLine = fileNames.ReadLine();
            }
            fileNames.Close();


            //gets the "database" of places entered
            StreamReader filePlaces = new StreamReader("customerPlaces.txt", false);
            count = 0;
            while ((fileLine = filePlaces.ReadLine()) != null)
            {
                if (fileLine != "")
                {
                    customerPlaces[count] = fileLine;
                    fileLinePlaces[count] = fileLine;
                    count++;
                }
            }
            filePlaces.Close();


            //Initialize and setup the map
            //**********************************************************************
            gmap1.ShowCenter = false;
            gmap1.MapProvider = GMap.NET.MapProviders.GoogleMapProvider.Instance;
            GMap.NET.GMaps.Instance.Mode = GMap.NET.AccessMode.ServerAndCache;

            gmap1.Zoom = 12;
            gmap1.DragButton = MouseButtons.Left;
            gmap1.SetPositionByKeywords("De Pere, Wisconsin");
            //**********************************************************************
        }

        //zoom in on the map when the zoom in icon is clicked
        private void pb_Zoomin_Click(object sender, EventArgs e)
        {
            if (gmap1.MaxZoom > gmap1.Zoom)
                gmap1.Zoom++;
        }

        //zoom out on the map when the zoom out icon is clicked
        private void pb_Zoomout_Click(object sender, EventArgs e)
        {
            if (gmap1.MinZoom < gmap1.Zoom)
                gmap1.Zoom--;
        }

        //Opens a new form to add a new customer to the map
        private void btn_AddCustomer_Click(object sender, EventArgs e)
        {
            FormAddCustomer child = new FormAddCustomer(customerArray, customerPlaces);  //i
            DialogResult dr = child.ShowDialog();

            if (dr == DialogResult.OK)    //if submit button clicked, add customer to array
            {
                for (int i = 0; i < customerArray.Length; i++)
                {
                    if (customerArray[i].name == "")    //add to next "free" spot in array
                    {
                        //transfer over information from the addCustomer form
                        customerArray[i].name = child.firstname + " " + child.lastname;
                        customerArray[i].wheelchair = child.hasWheelchair;
                        customerArray[i].pickupAddr = child.pickup;
                        customerArray[i].dropoffAddr = child.dropoff;
                        customerArray[i].dropoffTime = child.timeDropoffEntered;
                        customerArray[i].pickupTime = child.timePickupEntered;

                        //Creates the pickup marker and adds it to the map
                        //pickup address - green
                        GeocoderLocation p = Geocoder.Locate(customerArray[i].pickupAddr);
                        GMarkerGoogle marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
                        //set id of marker                        
                        pickupMarkers.Id = "green";

                        PointLatLng point = marker.Position;
                        point.Lat = p.Latitude;
                        point.Lng = p.Longitude;
                        GDirections d;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        string address = d.StartAddress;    //address of marker added
                        customerArray[i].pickupAddr = address;  //add google calculated address to customer array

                        //set wheelchair information
                        string wheelchair = "no wheelchair";
                        if (customerArray[i].wheelchair == true)
                            wheelchair = "has wheelchair";

                        marker.ToolTipText = customerArray[i].name + "\n" + address + "\n" + wheelchair;

                        if (customerArray[i].pickupTime != "null")  //append pickup time if available
                        {
                            marker.ToolTipText += "\n" + customerArray[i].pickupTime;
                            address = customerArray[i].name + " | Pickup | " + address + " | " + customerArray[i].pickupTime;
                        }
                        else    
                            address = customerArray[i].name + " | Pickup | " + address;

                        marker.Tag = markerPairNum;  
                        
                        //custom font for tooltip
                        Font font = new Font(marker.ToolTip.Font.Name,9,FontStyle.Regular);     
                        marker.ToolTip.Font = font; 

                        customerArray[i].markerTag = markerPairNum;
                        pickupMarkers.Markers.Add(marker);
                        gmap1.Overlays.Add(pickupMarkers);

                        markerArray[i] = marker;

                        //adds marker to listbox
                        lb_Destinations.Items.Add(address);
                        gmap1.SetPositionByKeywords(address);   //update

                        //log add customer - pickup
                        for (int j = 0; j < 100; j++)
                        {
                            if (customerInteractions[j] == "")
                            {
                                customerInteractions[j] = "add customer; " + address;     //name | pickup/dropoff | addr (| time)
                                break;
                            }
                        }


                        //Creates the dropoff marker and adds it to the map - same as above, but for dropoff instead
                        //dropoff address - red
                        p = Geocoder.Locate(customerArray[i].dropoffAddr);
                        marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);

                        //set id of marker
                        dropoffMarkers.Id = "red";

                        point = marker.Position;
                        point.Lat = p.Latitude;
                        point.Lng = p.Longitude;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        address = d.StartAddress;    //address of marker added
                        customerArray[i].dropoffAddr = address;    //add google calculated address to customer array

                        marker.ToolTipText = customerArray[i].name + "\n" + address + "\n" + wheelchair;

                        string onlyAddress = address;

                        if (customerArray[i].dropoffTime != "null") //append dropoff time if available
                        {
                            marker.ToolTipText += "\n" + customerArray[i].dropoffTime;
                            address = customerArray[i].name + " | Dropoff | " + address + " | " + customerArray[i].dropoffTime;
                        }
                        else
                            address = customerArray[i].name + " | Dropoff | " + address;

                        marker.Tag = markerPairNum;
                        marker.ToolTip.Font = font; 

                        //add to the overlay and add to map
                        customerArray[i].markerTag = markerPairNum;
                        dropoffMarkers.Markers.Add(marker);
                        gmap1.Overlays.Add(dropoffMarkers);

                        markerArray[i + 1] = marker;

                        //add to the listbox
                        lb_Destinations.Items.Add(address);
                        gmap1.SetPositionByKeywords(onlyAddress);   //update

                        for (int j = 0; j < customerPlaces.Length; j++)     //add these addresses to the places array
                        {
                            if (customerPlaces[j] == "")
                            {
                                customerPlaces[j] = customerArray[i].dropoffAddr;
                                customerPlaces[j + 1] = customerArray[i].pickupAddr;
                                break;
                            }
                        }

                        //log add customer - addr
                        for (int j = 0; j < 100; j++)
                        {
                            if (customerInteractions[j] == "")
                            {
                                customerInteractions[j] = "add customer; " + address;     //name | pickup/dropoff | addr (| time)
                                break;
                            }
                        }

                        markerPairNum++;
                        break;
                    }
                }
            }
            if (lb_Destinations.Items.Count != 0)
                lb_Destinations.SelectedIndex = 0;
        }

        //catches right click or left click
        private void gmap1_OnMarkerClick(GMapMarker item, MouseEventArgs e)
        {
            //right click deletes selected marker from map and listbox
            if (e.Button == MouseButtons.Right)
            {
                pickupMarkers.Markers.Remove(item);    //clear directions, route, and remove marker
                dropoffMarkers.Markers.Remove(item);
                tb_Directions.Text = "";
                routesOverlay.Clear();
                gmap1.Overlays.Remove(routesOverlay);
                gmap1.ReloadMap();

                PointLatLng point = item.Position;
                point.Lat = item.Position.Lat;
                point.Lng = item.Position.Lng;

                GDirections d;
                GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                string address = d.StartAddress;    //address of marker clicked

                string[] name = item.ToolTipText.Split('\n');
                if (item.Overlay.Id == "green")
                    address = name[0] + " | Pickup | " + address;
                else
                    address = name[0] + " | Dropoff | " + address;



                //delete clicked marker address from destinations
                int index = 0;
                for (int i = 0; i < customerArray.Length; i++)
                {
                    for (int j = 0; j < lb_Destinations.Items.Count; j++)
                    {
                        if (lb_Destinations.Items[j].ToString().Contains(address))  //modified for containing time..
                        {
                            lb_Destinations.Items.Remove(lb_Destinations.Items[j]);
                            index = j;
                            break;
                        }
                    }
                }

                if (name.Length == 4)  //if a time was specified, append time
                    address += " | " + name[3];


                //add remove to log
                for (int i = 0; i < 100; i++)
                {
                    if (customerInteractions[i] == "")
                    {
                        customerInteractions[i] = "remove; " + address + "; " + index;     //name | pickup/dropoff | addr (| time)
                        break;
                    }
                }
            }

            //taxi location is updated to clicked marker location - taxi icon replaces marker, and entry is removed from the listbox
            if (e.Button == MouseButtons.Left)
            {
                routesOverlay.Clear();
                gmap1.Overlays.Remove(routesOverlay);
                PointLatLng point = item.Position;
                point.Lat = item.Position.Lat;
                point.Lng = item.Position.Lng;

                GDirections d;
                GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                string address = d.StartAddress;    //address of marker clicked
                string addressOnly = address;

                string[] name = item.ToolTipText.Split('\n');
                if (item.Overlay.Id == "green")
                    address = name[0].Trim() + " | Pickup | " + address;
                else
                    address = name[0].Trim() + " | Dropoff | " + address;

                if(name.Length==4)  //if a time was specified, append it
                    address += " | " + name[3];
                
                int index = 0;


                GMapMarkerImage customMarker = new GMapMarkerImage(point, taxiImage);    
                //put new marker for taxi on map, uses lat,lng from image
                
                    if (item.Overlay.Id == "green")   //if marker clicked == green
                    {
                        if (numPassengers < maxPassengers)  //checks to make sure you have enough room to pick up a customer
                        {
                            taxiMarker.Markers.Clear();
                            customMarker.IsHitTestVisible = false;
                            taxiMarker.Markers.Add(customMarker);
                            gmap1.Overlays.Add(taxiMarker);

                            //check if wheelchair
                            if (item.ToolTipText.Contains("has wheelchair") == true)
                            {
                                numPassengers += 2; //wheelchairs count as 2 spots
                                lab_NumWheelchairs.Text = (Convert.ToInt32(lab_NumWheelchairs.Text) + 1).ToString();
                            }
                            else
                                numPassengers++;  //update number of passengers in car

                            lab_NumPassengers.Text = numPassengers.ToString();
                            pickupMarkers.Markers.Remove(item); //customer has been picked up, remove from markers list
                            gmap1.Refresh();
                            gmap1.SetPositionByKeywords(addressOnly);
                            gmap1.ReloadMap();

                            //removes entry from listbox
                            for (int i = 0; i < lb_Destinations.Items.Count; i++)
                            {
                                if (lb_Destinations.Items[i].ToString() == address)
                                {
                                    index = i;
                                    break;
                                }
                            }
                            lb_Destinations.Items.Remove(address);
                            MBShown = false;
                        }
                        else //if not enough space, show an error
                        {
                            if (MBShown == false)
                            {
                                MessageBox.Show("You do not have enough space! Please drop off a customer first!");
                                MBShown = true;
                            }
                            return;
                        }
                    }


                    //taxi location is updated to clicked marker location - taxi icon replaces marker, and entry is removed from the listbox
                    if (item.Overlay.Id == "red")   //if marker clicked == red 
                    {
                        if (numPassengers > 0) //make sure there is at least 1 customer to dropoff
                        {
                            taxiMarker.Markers.Clear();
                            customMarker.IsHitTestVisible = false;
                            taxiMarker.Markers.Add(customMarker);
                            gmap1.Overlays.Add(taxiMarker);

                            //check if wheelchair and update passengers accordingly 
                            if (item.ToolTipText.Contains("has wheelchair") == true)
                            {
                                numPassengers -= 2;
                                if (numPassengers < 0)
                                {
                                    numPassengers = 0;
                                    lab_NumWheelchairs.Text = (Convert.ToInt32(lab_NumWheelchairs.Text)).ToString();
                                }
                                else
                                    lab_NumWheelchairs.Text = (Convert.ToInt32(lab_NumWheelchairs.Text) - 1).ToString();
                            }
                            else
                                numPassengers--;

                            lab_NumPassengers.Text = numPassengers.ToString();
                            dropoffMarkers.Markers.Remove(item);    //remove marker from marker list

                            gmap1.SetPositionByKeywords(addressOnly);

                            //get index of item to remove from listbox
                            for (int i = 0; i < lb_Destinations.Items.Count; i++)
                            {
                                if (lb_Destinations.Items[i].ToString() == address)
                                {
                                    index = i;
                                    break;
                                }
                            }
                            lb_Destinations.Items.Remove(address);
                            MBShown = false;
                        }
                        else
                        {
                            if (MBShown == false)
                            {
                                MessageBox.Show("You do not have any customers to drop off! Please pick up a customer first!");
                                MBShown = true;
                            }
                            return;
                        }
                    }

                    //write the name, pickup/dropoff, address, timestamp to a file to log  
                    for (int i = 0; i < 100; i++)
                    {
                        if (customerInteractions[i] == "")
                        {
                            customerInteractions[i] = "taxi; " + address + "; " + DateTime.Now.ToShortTimeString() + "; " + index;
                            break;
                        }
                    }
                    
                //clear route and directions data
                    if (lb_Destinations.Items.Count == 0)
                    {
                        routesOverlay.Clear();
                        tb_Directions.Text = "";
                        lab_MilesToDestData.Text = "0 miles";
                        lab_TimeToDestData.Text = "0 mins";
                    }                
            }
        }

        //function associated with the drag and drop listbox
        private void lb_Destinations_MouseDown(object sender, MouseEventArgs e)
        {
            if (this.lb_Destinations.SelectedItem == null) return;
            this.lb_Destinations.DoDragDrop(this.lb_Destinations.SelectedItem, DragDropEffects.Move);
            lb_Destinations.SelectedItem = -1;
        }

        //function associated with the drag and drop listbox
        private void lb_Destinations_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;

            gmap1.Overlays.Remove(routesOverlay);
            gmap1.Refresh();
            lb_Destinations.Text = "";
            tb_Directions.Text = "";
            lab_NumPassengers.Text = "0";
            lab_NumWheelchairs.Text = "0";
            lab_MilesToDestData.Text = "0 miles";
            lab_TimeToDestData.Text = "0 mins";
            lb_Destinations.ClearSelected();
        }
        //function associated with the drag and drop listbox
        private void lb_Destinations_DragDrop(object sender, DragEventArgs e)
        {
            Point point = lb_Destinations.PointToClient(new Point(e.X, e.Y));
            int index = this.lb_Destinations.IndexFromPoint(point);
            if (index < 0) index = this.lb_Destinations.Items.Count - 1;
            object data = e.Data.GetData(typeof(string));// (typeof(DateTime));
            this.lb_Destinations.Items.Remove(data);
            this.lb_Destinations.Items.Insert(index, data);
            lb_Destinations.ClearSelected();
        }

        //calculates the route based on the order of items in the list box
           //-gets distance, times, as well as turn by turn 
        private void btn_CalculateRoute_Click(object sender, EventArgs e)
        {
            //if (lb_Destinations.Items.Count == 0 || lb_Destinations.Items.Count == 1)
            //    MessageBox.Show("Please add at least 1 customer to the map first!");
            //else
            //{
                ep_appointmentAlert.SetError(lab_OrderofDest, "");
                //clear out old routes                
                routesOverlay.Clear();
                routesOverlay.Routes.Clear();
                gmap1.Overlays.Remove(routesOverlay);
                
                tb_Directions.Text = ""; //make sure directions TB is clear

                decimal miles = 0;
                int time = 0;   //total minutes for route
                int twoPointTime = 0; //time between 2 points
                string start;// = lb_Destinations.Items[0].ToString();
                string end;// = lb_Destinations.Items[0].ToString();

                //get rid of cust - pickup/dropoff
                string[] newaddress = lb_Destinations.Items[0].ToString().Split('|');
                //splits the string - name | pickup/dropoff | address (| req. time, if given)

                start = newaddress[2].Trim(); //index 2 should have only the address
                end = start;


                int numHours = 0;
                string[] timeStr;
                string appointmentTime = "null";
                if (newaddress.Length == 4)
                    appointmentTime = newaddress[3].Trim();

                int[] twoPointsArray = new int[100];
                for (int i = 0; i < 100; i++)   //intialize
                    twoPointsArray[i] = 0;

                //go through each item in listbox to compare to the previous, and add up directions, distance, and time
                foreach (var listBoxItem in lb_Destinations.Items)
                {
                    start = end;
                    newaddress = listBoxItem.ToString().Split('|'); 
                    end = newaddress[2].Trim();

                    if (taxiMarker.Markers.Count != 0)
                    {
                        PointLatLng point = taxiMarker.Markers[0].Position;

                        GDirections d;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        string address = d.StartAddress;    //address of marker clicked
                        start = address;
                    }

                    if (newaddress.Length == 4)
                    {
                        appointmentTime = newaddress[3].Trim();                     
                    }
                    else
                        appointmentTime = "null";
                    
                
                    if (start != end)
                    {
                        MapRoute route = GMap.NET.MapProviders.GoogleMapProvider.Instance.GetRoute(start, end, false, false, 15);

                        r = new GMapRoute(route.Points, "My route");

                        //gmap1. //clear out old routes
                        r.Stroke.Color = Color.Red; //change route color to red

                        //add route to map
                        routesOverlay.Routes.Add(r);
                        gmap1.Overlays.Add(routesOverlay);


                        GDirections direc = new GDirections();

                        //display miles from start to end
                        GMapProviders.GoogleMap.GetDirections(out direc, start, end, false, false, false, false, false);

                        //convert data to int, add, and then covert back to string to display
                        string[] milesStr = direc.Distance.Split(' ');
                        miles = miles + Convert.ToDecimal(milesStr[0]);
                        timeStr = direc.Duration.Split(' ');

                        //keep track of number of minutes
                        if (timeStr[1] == "hour" || timeStr[1] == "hours")
                        {
                            numHours = Convert.ToInt32(timeStr[0]);
                            time = time + (60 * numHours) + Convert.ToInt32(timeStr[2]);   //now all time in minutes
                            twoPointTime = (60 * numHours) + Convert.ToInt32(timeStr[2]);                           
                        }
                        else
                        {
                            time = time + Convert.ToInt32(timeStr[0]);
                            twoPointTime = Convert.ToInt32(timeStr[0]);
                        } 

                        lab_MilesToDestData.Text = miles.ToString() + " miles";

                        //show directions from start to dest.
                        for (int i = 0; i < direc.Steps.Count; i++)
                        {
                            var text = direc.Steps[i].ToString().Replace("<b>", "");
                            text = text.Replace("</b>", "");     //gets rid of some weird characters that come w/ directions
                            text = text.Replace("<div style=\"font-size:0.9em\">", "\n");
                            text = text.Replace("</div>", "");
                            tb_Directions.Text = tb_Directions.Text + text + "\n" + "\n";
                        }
                    }

                    //convert from minutes back to minutes and hours
                    numHours = 0;
                    while (true)
                    {
                        if (time > 60)
                        {
                            time = time - 60;
                            numHours++;
                        }
                        else
                            break;
                    }

                    //if  req time give, get the time assoicated with that address
                    if (appointmentTime != "null")
                    {
                        string pickup = "null";
                        string dropoff = "null";

                        for(int i = 0; i < lb_Destinations.Items.Count; i++)
                        {
                            if(lb_Destinations.Items[i].ToString().Contains(newaddress[0].Trim())) //&& lb_Destinations.Items[i].ToString().Contains(appointmentTime))
                            {
                                string[] array = lb_Destinations.Items[i].ToString().Split('|');
                                if(lb_Destinations.Items[i].ToString().Contains("Pickup"))                                                                 
                                    pickup = array[3].Trim();
                                else
                                    dropoff = array[3].Trim();                                
                            }
                        }

                        DateTime twoPoints = new DateTime();
                        twoPoints.AddMinutes(twoPointTime);
                        DateTime cur = DateTime.Now;
                        TimeSpan difference;
                        //"time" var is in minutes
                        difference = Convert.ToDateTime(appointmentTime) - cur;     //difference is time needed to get there

                        //newTime is actual time it will take to get there
                        //gets Time can converts to Datetime format
                        DateTime newTime = new DateTime();
                        newTime = newTime.AddHours(numHours);
                        newTime = newTime.AddMinutes(time);

                        TimeSpan x = newTime.TimeOfDay;

                        string pickupDropoff;
                        if (newaddress[1].Contains("Dropoff"))
                            pickupDropoff = "dropoff";
                        else
                            pickupDropoff = "pickup";
                        
                        //checks to see if the total route time is greater than the time needed to get there
                        //or if the time between two points is greater than the dropoff time - pickup time
                        //if so, it sets an alert to alert the user 
                        if(pickup!="null" && dropoff!="null")
                        {
                            TimeSpan reqTimeDiff = Convert.ToDateTime(dropoff) - Convert.ToDateTime(pickup);

                            if ((newTime.TimeOfDay) > difference || (twoPoints.TimeOfDay > reqTimeDiff))    // > means we will miss appt   || 
                                ep_appointmentAlert.SetError(lab_OrderofDest, "You are in danger of missing " + newaddress[0].Trim()
                                + "'s " + "requested " + pickupDropoff + " time!");
                        }
                    }
                }   //end foreach

                //now we should have correct number of hours with some value minutes less than 60, convert back to string                
                if (numHours == 0)
                    lab_TimeToDestData.Text = time.ToString() + " minutes";

                else if (numHours == 1)
                    lab_TimeToDestData.Text = numHours.ToString() + " hour " + time.ToString() + " minutes";

                else
                    lab_TimeToDestData.Text = numHours.ToString() + " hours " + time.ToString() + " minutes";


                newaddress = lb_Destinations.Items[0].ToString().Split('|');
                newaddress[2] = newaddress[2].Trim(); //index 2 should have only the address
                gmap1.SetPositionByKeywords(newaddress[2]);   //update now to show route            
        }

        //writes all "moves", places, and names entered to files on close
        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (testMode == true)
                return;

            //write all customer data to file on application close
            StreamWriter f = new StreamWriter("customerNames.txt", true);
            int count = 0;
            string fileLine;
            string wheelchair;

            while (customerArray[count].name != "") //all customer array spots that have data
            {
                bool inFile = false;
                if (customerArray[count].wheelchair == true)
                    wheelchair = "yes";
                else
                    wheelchair = "no";

                fileLine = customerArray[count].name + "; "  //customerArray[count].pickupAddr + "; " + customerArray[count].dropoffAddr
                    + wheelchair + "\n";

                for (int i = 0; i < customerArray.Length; i++) //write all customer info to file if name is not in file
                {
                    if (fileLine == fileLineInfo[i] || fileLine == fileLineInfo[i] + "\n" || fileLineInfo[i].Contains(customerArray[count].name))
                    {
                        inFile = true;
                        break;
                    }
                }

                if (inFile == false)
                    f.WriteLine(fileLine);

                count++;
            }
            f.Close();

            //write customer places to file
            StreamWriter fp = new StreamWriter("customerPlaces.txt", true);
            count = 0;
            while (customerPlaces[count] != "") //all customer array spots that have data
            {
                bool inFile = false;
                for (int i = 0; i < customerArray.Length; i++)
                {
                    if (customerPlaces[count] == fileLinePlaces[i])
                    {
                        inFile = true;
                        break;

                    }
                }
                if (inFile == false)
                {
                    if (fileLinePlaces[0] == "")
                        fp.WriteLine(customerPlaces[count] + "\n");
                    else
                        fp.WriteLine("\n" + customerPlaces[count]);
                }
                count++;
            }
            fp.Close();


            //adds all customerInteractions in a session to the file
            fp = new StreamWriter("customerInteractions.txt", true);
            count = 0;
            fp.WriteLine(DateTime.Today.ToShortDateString());
            while (customerInteractions[count] != "") //all customer array spots that have data
            {
                fp.WriteLine(customerInteractions[count]);
                count++;
            }
            fp.WriteLine();
            fp.Close();
        }

        //Clears all route, directions, and distance/time info on button click
        private void btn_ClearRoute_Click(object sender, EventArgs e)
        {
            routesOverlay.Clear();
            routesOverlay.Routes.Clear(); //.Remove(r);
            gmap1.Overlays.Remove(routesOverlay);
            gmap1.Refresh();
            lb_Destinations.Text = "";
            tb_Directions.Text = "";
            lab_NumPassengers.Text = "0";
            lab_NumWheelchairs.Text = "0";
            lab_MilesToDestData.Text = "0 miles";
            lab_TimeToDestData.Text = "0 mins";
            ep_appointmentAlert.SetError(lab_OrderofDest, "");
        }

        //clears all data from the map - same as above, but includes markers, the list box, taxi location
        private void btn_ClearMarkers_Click(object sender, EventArgs e)
        {
            routesOverlay.Clear();
            gmap1.Overlays.Clear();
            pickupMarkers.Clear();
            dropoffMarkers.Clear();

            gmap1.Refresh();
            lb_Destinations.Text = "";
            tb_Directions.Text = "";
            lab_NumPassengers.Text = "0";
            lab_NumWheelchairs.Text = "0";
            lab_MilesToDestData.Text = "0 miles";
            lab_TimeToDestData.Text = "0 mins";
            ep_appointmentAlert.SetError(lab_OrderofDest, "");
            lb_Destinations.Items.Clear();
        }

        //closes the app on "quit"
        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        //fires when the clear route tool strip button is clicked - calls the previous clearRoute function
        private void clearRouteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_ClearRoute_Click(sender, e);
        }

        //events for menu strip items - use same events as buttons
        private void addCustomerToMapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_AddCustomer_Click(sender, e);
        }

        //event for clear all data tool strip menu item - same event is the form button
        private void clearMarkersAndRouteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_ClearMarkers_Click(sender, e);
        }
    //handles the right click on a listbox item - brings up a context menu strip that allows for deleting the selected item, or
        //that customer
        private void lb_Destinations_MouseUp(object sender, MouseEventArgs e)
        {
            if (this.lb_Destinations.SelectedItem == null)
                return;

            if (e.Button == MouseButtons.Right)
            {
                var item = lb_Destinations.IndexFromPoint(e.Location);
                if (item >= 0)
                {
                    lb_Destinations.SelectedIndex = item;
                    CMS_TB.Show(lb_Destinations, e.Location);
                }                
            }
        }

        //item on the context menu strip above - removes the selected item in the listbox
        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
                string selected = lb_Destinations.SelectedItem.ToString();

                string[] place = selected.Split('|');
                string selectedNew = place[2].Trim();

                GeocoderLocation p = Geocoder.Locate(selectedNew);
                GMarkerGoogle markerGreen = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
                GMarkerGoogle markerRed = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
                markerGreen.ToolTipText = selectedNew;
                markerRed.ToolTipText = selectedNew;
                
                //gets the data from the selected item in list box and finds the matching marker to delete
                if(place[1] == " Pickup ")
                    foreach (GMarkerGoogle m in pickupMarkers.Markers)
                    {
                        if (m.ToolTipText.Contains(place[2].Trim()) && m.ToolTipText.Contains(place[0].Trim()))
                        {                   
                            pickupMarkers.Markers.Remove(m);
                            break;
                        }
                    }
                else
                     foreach (GMarkerGoogle m in dropoffMarkers.Markers)    //has same name and location
                    {
                        if (m.ToolTipText.Contains(place[2].Trim()) && m.ToolTipText.Contains(place[0].Trim()))
                        {// == place[0] + place[1] + place[2])
                            dropoffMarkers.Markers.Remove(m);
                            break;
                        }
                    }
                
            //set time, distance, and directions to 0,
                tb_Directions.Text = "";
                lab_MilesToDestData.Text = "0 miles";
                lab_TimeToDestData.Text = "0 mins";
                ep_appointmentAlert.SetError(lab_OrderofDest, "");
                routesOverlay.Clear();
                gmap1.Overlays.Remove(routesOverlay);
                gmap1.ReloadMap();


                //finally delete clicked item from destinations listbox
                int index = 0;
                for (int i = 0; i < customerArray.Length; i++)
                {
                    for (int j = 0; j < lb_Destinations.Items.Count; j++)
                    {
                        if (lb_Destinations.Items[j].ToString() == selected)
                        {
                            index = j;
                            lb_Destinations.Items.Remove(lb_Destinations.Items[j]);
                        }
                    }
                }
                //highlight the first item in listbox if one is available
                if(lb_Destinations.Items.Count!=0)
                    lb_Destinations.SelectedIndex = 0;


                //add remove item to log
                for (int i = 0; i < 100; i++)
                {
                    if (customerInteractions[i] == "")
                    {
                        customerInteractions[i] = "remove; " + selected + "; " + index;     //name | pickup/dropoff | addr (| time)
                        break;
                    }
                }
        }
        //same as above, but finds the associated pickup/dropoff item and marker as well
        private void removeToolStripMenuItemBoth_Click(object sender, EventArgs e)
        {
            string selected = lb_Destinations.SelectedItem.ToString();

            string[] place = selected.Split('|');
            string selectedNew = place[2].Trim();

            GeocoderLocation p = Geocoder.Locate(selectedNew);
            GMarkerGoogle markerGreen = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
            GMarkerGoogle markerRed = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
            markerGreen.ToolTipText = selectedNew;
            markerRed.ToolTipText = selectedNew;


            //find the two address and name that matches
            int index = 0;
            for (int i = 0; i < 100; i++)
            {
                if (selected.Contains(customerArray[i].name) && ((customerArray[i].pickupAddr == "") || (customerArray[i].dropoffAddr == "")))
                {
                    i++;
                }
                if (selected.Contains(customerArray[i].name) && (selected.Contains(customerArray[i].pickupAddr) || selected.Contains(customerArray[i].dropoffAddr)))
                {
                    index = i;
                    break;
                }
            }
            //search the marker arrays and delete the markers
                foreach (GMarkerGoogle m in pickupMarkers.Markers)
                {                  
                    if (m.ToolTipText.Contains(place[0].Trim()) && Convert.ToInt32(m.Tag)==customerArray[index].markerTag)
                    {                   
                        pickupMarkers.Markers.Remove(m);
                        break;
                    }
                }

                foreach (GMarkerGoogle m in dropoffMarkers.Markers)    //has same name and location
                {
                    if (m.ToolTipText.Contains(place[0].Trim()) && Convert.ToInt32(m.Tag) == customerArray[index].markerTag)
                    {// == place[0] + place[1] + place[2])
                        dropoffMarkers.Markers.Remove(m);
                        break;
                    }
                }
            //clear out directions, time, distance, and routes
                tb_Directions.Text = ""; 
                lab_MilesToDestData.Text = "0 miles";
                lab_TimeToDestData.Text = "0 mins";
                ep_appointmentAlert.SetError(lab_OrderofDest, "");
            routesOverlay.Clear();
            gmap1.Overlays.Remove(routesOverlay);
            gmap1.ReloadMap();

            //find exact two items to delete from the listbox
            int indexP = 0;
            int indexD = 0;
            for (int j = 0; j < lb_Destinations.Items.Count; j++)
            {
                if (lb_Destinations.Items[j].ToString().Contains(customerArray[index].pickupAddr) && lb_Destinations.Items[j].ToString().Contains(customerArray[index].name))
                {
                    indexP = j;
                    break;
                }
            }
            for (int j = 0; j < lb_Destinations.Items.Count; j++)
            {
                if (lb_Destinations.Items[j].ToString().Contains(customerArray[index].dropoffAddr) && lb_Destinations.Items[j].ToString().Contains(customerArray[index].name))
                {
                    indexD = j;
                    break;
                }
            }

            selected = lb_Destinations.Items[indexP].ToString();
            string selectedD = lb_Destinations.Items[indexD].ToString();

            //remove the items from listbox
            lb_Destinations.Items.Remove(selected);
            lb_Destinations.Items.Remove(selectedD);

            //log the removes in customerInteractions array
            for (int j = 0; j < 100; j++)
            {
                if (customerInteractions[j] == "")
                {
                    customerInteractions[j] = "remove both; " + selected + "; " + indexP;     //name | pickup/dropoff | addr (| time)
                    customerInteractions[j + 1] = "remove both; " + selectedD + "; " + indexD;
                    break;      //indexP and indexD hold the indices that they were removes from - used for undo so they can be inserted 
                                //back into same indices
                }
            }
            
            if(lb_Destinations.Items.Count!=0)
                lb_Destinations.SelectedIndex = 0;           
        }

        //opens the Help form and makes sure only 1 can be open at a time
        private void howToUseToolStripMenuItem_Click(object sender, EventArgs e)
        {            
            if (childHelp == null || childHelp.isOpen==false)
            {
                childHelp = new HelpWindow();
                childHelp.Show();                
            }
            else
            {
                childHelp.BringToFront();                
            }            
        }
        //addCustomer tool strip menu item - calls the same function the form button does
        private void addCustomerToMapToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            btn_AddCustomer_Click(sender, e);
        }

        //has the ability to undo a customer addition, removing a marker, removing a customer, and picking up or dropping off a customer
        private void undoLastMoveToolStripMenuItem_Click(object sender, EventArgs e)        //remove marker, pickup/dropoff, add cust, remove marker set
        {
            string move = "";
            int interactionIndex = 0;
            //if no moves to undo, display a message. Otherwise, get the last move that was done
            for (int i = 0; i < 100; i++)
            {
                if (customerInteractions[i] == "" && i == 0)
                {
                    MessageBox.Show("There is nothing to undo!");
                    break;
                }
                if (customerInteractions[i] == "")
                {
                    move = customerInteractions[i - 1];
                    customerInteractions[i - 1] = "";
                    interactionIndex = i-1; //index of the last move
                    routesOverlay.Clear();
                    gmap1.Overlays.Remove(routesOverlay);
                    break;
                }
            }

            //if move is found, check what kind it was
            if (move != "")
            {
                string[] newMove;

                if (move.Contains("remove;"))    //undo marker right click (delete)
                {
                    newMove = move.Split(';');       //0 is "remove; "      remove; name | pickup/dropoff | addr (| time)
                    move = newMove[1].Trim();        
                    string markerColor;
                    int moveIndex = Convert.ToInt32(newMove[2].Trim());

                    //separate all information from the move
                    if (move.Contains("Pickup"))
                        markerColor = "green";
                    else
                        markerColor = "red";

                    string[] info = move.Split('|'); //name, pickup/dropoff, addr, (time)

                    GeocoderLocation p = Geocoder.Locate(info[2]);
                    string wheelchair = "no wheelchair";
                    int index = 0;

                    for (int i = 0; i < 100; i++)
                    {
                        if (customerArray[i].name == info[0].Trim() && (customerArray[i].pickupAddr == info[2].Trim() || customerArray[i].dropoffAddr == info[2].Trim()))
                        {
                            if (customerArray[i].wheelchair == true)
                                wheelchair = "has wheelchair";
                            index = i;
                            break;

                        }
                    }

                    //if pickup marker, create a new marker and add back to the marker array, listbox, and map
                    if (markerColor == "green")
                    {
                        GMarkerGoogle marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
                        pickupMarkers.Id = "green";

                        PointLatLng point = marker.Position;
                        point.Lat = p.Latitude;
                        point.Lng = p.Longitude;
                        GDirections d;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        string address = d.StartAddress;    //address of marker added
                        // customerArray[i].pickupAddr = address;  //add google calculated address to customer array

                        address = info[2].Trim();
                        marker.ToolTipText = customerArray[index].name + "\n" + address + "\n" + wheelchair;

                        if (customerArray[index].pickupTime != "null")
                        {
                            marker.ToolTipText += "\n" + customerArray[index].pickupTime;
                            address = customerArray[index].name + " | Pickup | " + address + " | " + customerArray[index].pickupTime;
                        }
                        else    //add pickuptime to LB if available
                            address = customerArray[index].name + " | Pickup | " + address;


                        pickupMarkers.Markers.Add(marker);
                        gmap1.Overlays.Add(pickupMarkers);

                        markerArray[index] = marker;

                        lb_Destinations.Items.Insert(moveIndex, address);
                        gmap1.SetPositionByKeywords(address);   //update
                    }

                    //if dropoff marker, create a new marker and add back to the marker array, listbox, and map
                    else
                    {
                        GMarkerGoogle marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
                        dropoffMarkers.Id = "red";

                        PointLatLng point = marker.Position;
                        point.Lat = p.Latitude;
                        point.Lng = p.Longitude;
                        GDirections d;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        string address = d.StartAddress;    //address of marker added
                        // customerArray[i].pickupAddr = address;  //add google calculated address to customer array



                        address = info[2].Trim();
                        marker.ToolTipText = customerArray[index].name + "\n" + address + "\n" + wheelchair;

                        if (customerArray[index].dropoffTime != "null")
                        {
                            marker.ToolTipText += "\n" + customerArray[index].dropoffTime;
                            address = customerArray[index].name + " | Dropoff | " + address + " | " + customerArray[index].dropoffTime;
                        }
                        else    //add dropoff time to LB if available
                            address = customerArray[index].name + " | Dropoff | " + address;


                        dropoffMarkers.Markers.Add(marker);
                        gmap1.Overlays.Add(dropoffMarkers);

                        markerArray[index] = marker;

                        lb_Destinations.Items.Insert(moveIndex, address);
                        gmap1.SetPositionByKeywords(address);   //update
                        return;
                    }
                }


                 //undo marker left click
                if(move.Contains("taxi"))
                {
                    newMove = move.Split(';');  
                    move = newMove[1].Trim();   //2 is time that it occured
                                                //name | pickupdropoff | addr (| time)
                    //get move info
                    string markerColor;

                    if (move.Contains("Pickup"))
                        markerColor = "green";
                    else
                        markerColor = "red";

                    string[] info = move.Split('|'); //name, pickup/dropoff, addr, (time)
                    int moveIndex = Convert.ToInt32(newMove[3]);

                    GeocoderLocation p = Geocoder.Locate(info[2]);  //get coordinates from address
                    string wheelchair = "no wheelchair";
                    int index = 0;

                    //re adjust number of passengers and wheelchairs
                    for (int i = 0; i < 100; i++)
                    {
                        if (customerArray[i].name == info[0].Trim() && (customerArray[i].pickupAddr.Contains(info[2].Trim()) || customerArray[i].dropoffAddr.Contains(info[2].Trim())))
                        {
                            if (customerArray[i].wheelchair == true)
                            {
                                wheelchair = "has wheelchair";  //modify num passengers
                                if (markerColor == "red")
                                {
                                    numPassengers += 2;
                                    lab_NumWheelchairs.Text = (Convert.ToInt32(lab_NumWheelchairs.Text) + 1).ToString();
                                }
                                else
                                {
                                    numPassengers -= 2;
                                    lab_NumWheelchairs.Text = (Convert.ToInt32(lab_NumWheelchairs.Text) - 1).ToString();
                                }
                            }
                            else
                            {
                                if (markerColor == "red")
                                    numPassengers++;
                                else
                                    numPassengers--;
                            }

                            index = i;
                            break;
                        }
                    }
          
                    GMarkerGoogle marker;// = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
                    string address;
                    string addrOnly; //

                    //if a dropoff marker was clicked, delete the taxi and put the marker back 
                    if (markerColor == "red")
                    {
                        marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
                        PointLatLng point = marker.Position;
                        point.Lat = marker.Position.Lat;
                        point.Lng = marker.Position.Lng;

                        GDirections d;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        address = d.StartAddress;    //address of marker clicked
                        addrOnly = address;

                        marker.ToolTipText = customerArray[index].name + "\n" + address + "\n" + wheelchair;
                        if (customerArray[index].dropoffTime != "null")
                        {
                            marker.ToolTipText += "\n" + customerArray[index].dropoffTime;
                            address = customerArray[index].name + " | Dropoff | " + address + " | " + customerArray[index].dropoffTime;
                        }
                        else    //add pickuptime to LB if available
                            address = customerArray[index].name + " | Dropoff | " + address;

                        customerArray[index].markerTag = markerPairNum;
                        marker.Tag = markerPairNum;  

                        Font font = new Font(marker.ToolTip.Font.Name, 9, FontStyle.Regular);
                        marker.ToolTip.Font = font; 
                        dropoffMarkers.Markers.Add(marker);
                        gmap1.Overlays.Add(dropoffMarkers);
                    }
                        //remove taxi and put the pickup marker back
                    else
                    {
                        marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
                        PointLatLng point = marker.Position;
                        point.Lat = marker.Position.Lat;
                        point.Lng = marker.Position.Lng;

                        GDirections d;
                        GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                        address = d.StartAddress;    //address of marker clicked
                        addrOnly = address;

                        marker.ToolTipText = customerArray[index].name + "\n" + address + "\n" + wheelchair;
                        if (customerArray[index].pickupTime != "null")
                        {
                            marker.ToolTipText += "\n" + customerArray[index].pickupTime;
                            address = customerArray[index].name + " | Pickup | " + address + " | " + customerArray[index].pickupTime;
                        }
                        else    //add pickuptime to LB if available
                            address = customerArray[index].name + " | Pickup | " + address;

                        customerArray[index].markerTag = markerPairNum;
                        marker.Tag = markerPairNum;  

                        Font font = new Font(marker.ToolTip.Font.Name, 9, FontStyle.Regular);
                        marker.ToolTip.Font = font; 
                        pickupMarkers.Markers.Add(marker);
                        gmap1.Overlays.Add(pickupMarkers);
                    }

                    //get last taxi position and add back to map
                    PointLatLng lastPoint = new PointLatLng(0,0); //lastPoint.Lat = 0; lastPoint.Lng = 0;
                    bool lastPointExists = false;                    
                    taxiMarker.Markers.Clear();

                    string lastAddress = "";
                    for(int i = interactionIndex-1; i > 0; i--) //count backwords to find last taxi location
                    {
                         if(customerInteractions[i].Contains("taxi"))
                         {
                             lastAddress = info[2];
                             string[] taxiInfo = customerInteractions[i].Split(';');
                             taxiInfo[1] = taxiInfo[1].Trim();
                             taxiInfo = taxiInfo[1].Split('|'); //taxiInfo[2] now has addr

                             string lastTaxtAddr = taxiInfo[2].Trim();
                             p = Geocoder.Locate(lastTaxtAddr);
                             marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
                             lastPoint = marker.Position;
                             lastPointExists = true;
                             break;
                         }
                    }
                    //only add if there was a last taxi position
                    if (lastPointExists == true)
                    {
                        GMapMarkerImage customMarker = new GMapMarkerImage(lastPoint, taxiImage);
                        customMarker.IsHitTestVisible = false;
                        taxiMarker.Markers.Add(customMarker);
                        gmap1.Overlays.Add(taxiMarker);
                        gmap1.Refresh();
                    }

                    lab_NumPassengers.Text = numPassengers.ToString(); 
                    lb_Destinations.Items.Insert(moveIndex, address);
                    gmap1.SetPositionByKeywords(address);
                    return;
                }

                    //undo the add customer function
                else if(move.Contains("add customer"))   //move and the previous
                {
                    markerPairNum--;
                    string prevMove = customerInteractions[interactionIndex - 1];
                    //move and prevMove contain the 2 moves added for the customer

                    newMove = move.Split(';');       //add customer; ...
                    move = newMove[1].Trim();        
                    newMove = prevMove.Split(';');
                    prevMove = newMove[1].Trim();

                    string[] info = move.Split('|'); //name, pickup/dropoff, addr, (time)
                    string[] prevInfo = prevMove.Split('|');

                    //go through the marker arrays, search for the corresponding markers and delete them
                    foreach (GMarkerGoogle m in pickupMarkers.Markers)
                    {
                        if (m.ToolTipText.Contains(prevInfo[2].Trim()) && m.ToolTipText.Contains(prevInfo[0].Trim()))
                        {                   
                            pickupMarkers.Markers.Remove(m);
                            break;
                        }
                    }

                    foreach (GMarkerGoogle m in dropoffMarkers.Markers)    //has same name and location
                    {
                        if (m.ToolTipText.Contains(info[2].Trim()) && m.ToolTipText.Contains(info[0].Trim()))
                        {
                            dropoffMarkers.Markers.Remove(m);
                            break;
                        }
                    }

                    //reset info
                    tb_Directions.Text = "";
                    lab_MilesToDestData.Text = "0 miles";
                    lab_TimeToDestData.Text = "0 mins";
                    ep_appointmentAlert.SetError(lab_OrderofDest, "");
                    gmap1.Overlays.Remove(routesOverlay);
                    gmap1.ReloadMap();

                    int foundBoth = 0;
                    //delete both customer markers from the listbox
                    for (int i = 0; i < customerArray.Length; i++)
                    {
                        for (int j = 0; j < lb_Destinations.Items.Count; j++)
                        {
                            if (lb_Destinations.Items[j].ToString() == move)
                            {
                                lb_Destinations.Items.Remove(lb_Destinations.Items[j]);
                                foundBoth++;
                            }
                            if (foundBoth == 2)
                                break;

                            if (lb_Destinations.Items.Count == 1)                            
                                j = 0;

                            if (lb_Destinations.Items[j].ToString() == prevMove)
                            {
                                lb_Destinations.Items.Remove(lb_Destinations.Items[j]);
                                foundBoth++;
                            }
                            if (foundBoth == 2)
                                break;                            
                        }
                        if (foundBoth == 2)
                            break;
                    }

                    if (lb_Destinations.Items.Count != 0)
                        lb_Destinations.SelectedIndex = 0;

                    for (int i = 0; i < 100; i++)   //also clears out prevMove - makes sure both "add customer" moves are cleared
                    {
                        if (customerInteractions[i] == "")
                        {
                            customerInteractions[i - 1] = "";
                            break;
                        }
                    }
                }
                
                //undos the removing of both a customer's markers
                else if (move.Contains("remove both"))
                {                    
                    //use move and prevMove - last 2 moves
                    string prevMove = customerInteractions[interactionIndex - 1];
                    newMove = move.Split(';');       
                    move = newMove[1].Trim();       //remove both; ... - dropoff (gets only the addr)
                    int moveIndex = Convert.ToInt32(newMove[2].Trim()); //indices contain where from the listbox they were deleted
                    newMove = prevMove.Split(';');  
                    prevMove = newMove[1].Trim();   // - pickup addr
                    int prevMoveIndex = Convert.ToInt32(newMove[2].Trim());
   
                    string[] info = move.Split('|'); //name, pickup/dropoff, addr, (time)
                    string[] prevInfo = prevMove.Split('|');


                    //add green marker back
                    GeocoderLocation p = Geocoder.Locate(prevInfo[2]);
                    string wheelchair = "no wheelchair";
                    int index = 0;

                    //search the array to get wheelchair info
                    for (int i = 0; i < 100; i++)
                    {
                        if (customerArray[i].name == prevInfo[0].Trim() && (customerArray[i].pickupAddr == prevInfo[2].Trim() || customerArray[i].dropoffAddr == prevInfo[2].Trim()))
                        {
                            if (customerArray[i].wheelchair == true)
                                wheelchair = "has wheelchair";
                            index = i;
                            break;
                        }
                    }
                    //create new marker and give it data
                    GMarkerGoogle marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
                    pickupMarkers.Id = "green";
                   
                    PointLatLng point = marker.Position;
                    point.Lat = p.Latitude;
                    point.Lng = p.Longitude;
                    GDirections d;
                    GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                    string address = d.StartAddress;    //address of marker added

                    address = prevInfo[2].Trim();
                    marker.ToolTipText = customerArray[index].name + "\n" + address + "\n" + wheelchair;

                    if (customerArray[index].pickupTime != "null")
                    {
                        marker.ToolTipText += "\n" + customerArray[index].pickupTime;
                        address = customerArray[index].name + " | Pickup | " + address + " | " + customerArray[index].pickupTime;
                    }
                    else    //add pickuptime to LB if available
                        address = customerArray[index].name + " | Pickup | " + address;

                    marker.Tag = markerPairNum;
                    Font font = new Font(marker.ToolTip.Font.Name, 9, FontStyle.Regular);
                    marker.ToolTip.Font = font; 

                    for(int i = 0; i < 100; i++)    //reset markertag
                    {
                        if (address.Contains(customerArray[i].name) && address.Contains(customerArray[i].pickupAddr))
                        {
                            customerArray[i]
                                .markerTag = markerPairNum;
                            break;
                        }
                    }
                    pickupMarkers.Markers.Add(marker);
                    gmap1.Overlays.Add(pickupMarkers);

                    markerArray[index] = marker;

                    lb_Destinations.Items.Insert(prevMoveIndex,address);
                    gmap1.SetPositionByKeywords(address);   //update

                    
                    
                    //red marker
                    p = Geocoder.Locate(info[2]);
                    //prevInfo
                    for (int i = 0; i < 100; i++)
                    {
                        if (customerArray[i].name == info[0].Trim() && (customerArray[i].pickupAddr == info[2].Trim() || customerArray[i].dropoffAddr == info[2].Trim()))
                        {
                            if (customerArray[i].wheelchair == true)
                                wheelchair = "has wheelchair";
                            index = i;
                            break;
                        }
                    }

                    marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);
                    dropoffMarkers.Id = "red";

                    point = marker.Position;
                    point.Lat = p.Latitude;
                    point.Lng = p.Longitude;                        
                    GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                    address = d.StartAddress;    //address of marker added

                    address = info[2].Trim();
                    marker.ToolTipText = customerArray[index].name + "\n" + address + "\n" + wheelchair;

                    if (customerArray[index].dropoffTime != "null")
                    {
                        marker.ToolTipText += "\n" + customerArray[index].dropoffTime;
                        address = customerArray[index].name + " | Dropoff | " + address + " | " + customerArray[index].dropoffTime;
                    }
                    else    //add dropoff time to LB if available
                        address = customerArray[index].name + " | Dropoff | " + address;

                    marker.Tag = markerPairNum;

                    marker.ToolTip.Font = font; 

                    for (int i = 0; i < 100; i++)    //reset markertag
                    {
                        if (address.Contains(customerArray[i].name) && address.Contains(customerArray[i].dropoffAddr))
                        {
                            customerArray[i].markerTag = markerPairNum;
                            break;
                        }
                    }
                    dropoffMarkers.Markers.Add(marker);
                    gmap1.Overlays.Add(dropoffMarkers);

                    markerArray[index] = marker;

                    lb_Destinations.Items.Insert(moveIndex, address);
                    gmap1.SetPositionByKeywords(address);   //update                             
                }
                markerPairNum++;
                customerInteractions[interactionIndex - 1] = "";
            }
        }

        //Loads 3 hardcoded customers to make testing easy - simulates the "add customer" function, but doesn't actually call it
        private void loadDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            testMode = true;

            DateTime time = new DateTime(); 
            DateTime cur = new DateTime();
            time = DateTime.Now;
            cur = DateTime.Now;
            cur = cur.AddHours(1);
            time = cur.AddMinutes(30);

            customerArray[0].name = "Alex Beltz";
            customerArray[0].wheelchair = false;
            customerArray[0].pickupAddr = "705 Fourth Street, De Pere, WI";
            customerArray[0].dropoffAddr = "100 Grant Street, De Pere, WI";
            customerArray[0].dropoffTime = time.ToShortTimeString();
            customerArray[0].pickupTime = cur.ToShortTimeString();
            customerArray[0].markerTag = markerPairNum;

            time = DateTime.Now;
            cur = DateTime.Now;
            cur = cur.AddMinutes(15);
            time = cur.AddMinutes(30);
            markerPairNum++;

            customerArray[1].name = "Isabella Beltz";
            customerArray[1].wheelchair = true;
            customerArray[1].pickupAddr = "300 Grant Street, De Pere, WI";
            customerArray[1].dropoffAddr = "1265 Lombardi Ave, Green Bay, WI 54304";
            customerArray[1].dropoffTime = time.ToShortTimeString();
            customerArray[1].pickupTime = cur.ToShortTimeString();// "12:00 PM";
            customerArray[1].markerTag = markerPairNum;

            time = DateTime.Now;
            cur = DateTime.Now;
            cur = cur.AddHours(1);
            cur = cur.AddMinutes(30);
            time = cur.AddMinutes(30);
            markerPairNum++;

            customerArray[2].name = "Kelly Beltz";
            customerArray[2].wheelchair = false;
            customerArray[2].pickupAddr = "136 Marsh Street, De Pere, WI";
            customerArray[2].dropoffAddr = "1001 Main Ave, De Pere, WI";
            customerArray[2].dropoffTime = time.ToShortTimeString();
            customerArray[2].pickupTime = cur.ToShortTimeString();
            customerArray[2].markerTag = markerPairNum;
           

            for(int i =0; i < 3; i++)
            {
                //pickup address - green
                GeocoderLocation p = Geocoder.Locate(customerArray[i].pickupAddr);
                GMarkerGoogle marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.green_dot);
                //set id of marker
                pickupMarkers.Id = "green";

                PointLatLng point = marker.Position;
                point.Lat = p.Latitude;
                point.Lng = p.Longitude;
                GDirections d;
                GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                string address = d.StartAddress;    //address of marker added
                customerArray[i].pickupAddr = address;  //add google calculated address to customer array

                string wheelchair = "no wheelchair";

                if (customerArray[i].wheelchair == true)
                    wheelchair = "has wheelchair";

                marker.ToolTipText = customerArray[i].name + "\n" + address + "\n" + wheelchair;

                if (customerArray[i].pickupTime != "null")
                {
                    marker.ToolTipText += "\n" + customerArray[i].pickupTime;
                    address = customerArray[i].name + " | Pickup | " + address + " | " + customerArray[i].pickupTime;
                }
                else    //add pickuptime to LB if available
                    address = customerArray[i].name + " | Pickup | " + address;

                marker.Tag = markerPairNum;  
                Font font = new Font(marker.ToolTip.Font.Name, 9, FontStyle.Regular);
                marker.ToolTip.Font = font; 
                customerArray[i].markerTag = markerPairNum;
                pickupMarkers.Markers.Add(marker);
                gmap1.Overlays.Add(pickupMarkers);

                markerArray[i] = marker;
                       
                lb_Destinations.Items.Add(address);
                gmap1.SetPositionByKeywords(address);   //update

                //log add customer - pickup
                for (int j = 0; j < 100; j++)
                {
                    if (customerInteractions[j] == "")
                    {
                        customerInteractions[j] = "add customer; " + address;     //name | pickup/dropoff | addr (| time)
                        break;
                    }
                }


                //dropoff address - red
                p = Geocoder.Locate(customerArray[i].dropoffAddr);
                marker = new GMarkerGoogle(new PointLatLng(p.Latitude, p.Longitude), GMarkerGoogleType.red_dot);

                //set id of marker
                dropoffMarkers.Id = "red";

                point = marker.Position;
                point.Lat = p.Latitude;
                point.Lng = p.Longitude;
                GMapProviders.GoogleMap.GetDirections(out d, point, point, false, false, false, false, false);
                address = d.StartAddress;    //address of marker added
                customerArray[i].dropoffAddr = address;    //add google calculated address to customer array

                marker.ToolTipText = customerArray[i].name + "\n" + address + "\n" + wheelchair;

                string onlyAddress = address;

                if (customerArray[i].dropoffTime != "null")
                {
                    marker.ToolTipText += "\n" + customerArray[i].dropoffTime;
                    address = customerArray[i].name + " | Dropoff | " + address + " | " + customerArray[i].dropoffTime;
                }
                else
                    address = customerArray[i].name + " | Dropoff | " + address;

                marker.Tag = markerPairNum; 
                marker.ToolTip.Font = font; 
                customerArray[i].markerTag = markerPairNum;
                dropoffMarkers.Markers.Add(marker);
                gmap1.Overlays.Add(dropoffMarkers);

                markerArray[i + 1] = marker;
                      
                lb_Destinations.Items.Add(address);
                gmap1.SetPositionByKeywords(onlyAddress);   //update

                for (int j = 0; j < customerPlaces.Length; j++)     //add these addresses to the places array
                {
                    if (customerPlaces[j] == "")
                    {
                        customerPlaces[j] = customerArray[i].dropoffAddr;
                        customerPlaces[j + 1] = customerArray[i].pickupAddr;
                        break;
                    }
                }

                //log add customer - addr
                for (int j = 0; j < 100; j++)
                {
                    if (customerInteractions[j] == "")
                    {
                        customerInteractions[j] = "add customer; " + address;     //name | pickup/dropoff | addr (| time)
                        break;
                    }
                }
            }
        }
        //********* problems with OnMarkerEnter and OnMarkerLeave - crashes for some reason on leave, could not figure out how to 
        //cirrectly makes this work without an error appearing.
        //if both functions are uncommented, the code "works", but there is an error. You can hit continue and the app will continue to
        //run perfectly

         //highlight moueover marker and corresponding pickup/dropoff marker
        //[MethodImpl(MethodImplOptions.Synchronized)]
        private void gmap1_OnMarkerEnter(GMapMarker item)
        {
            //try{
            //    if (markerOverOnce == false)
            //    {
            //        string[] markerData = item.ToolTipText.Split('\n');     //name, address, wheelchair, (time)
            //        string name = markerData[0].Trim();
            //        string addr = markerData[1].Trim();
            //        string tag = item.Tag.ToString();

            //        GMapMarker newMarker = new GMarkerGoogle(item.Position, GMarkerGoogleType.blue_dot);

            //        newMarker.Tag = tag;
            //        newMarker.ToolTipText = item.ToolTipText;
            //        newMarker.Position = item.Position;

            //        newMarker.IsVisible = true;
            //        mouseOverMarkers.Markers.Add(newMarker);

            //        ////make an additional marker and place on top?


            //        foreach (GMarkerGoogle m in dropoffMarkers.Markers.ToList())
            //        {
            //            if (m.Tag.ToString() == tag)
            //            {
            //                newMarker = new GMarkerGoogle(m.Position, GMarkerGoogleType.blue_dot);
            //                newMarker.Tag = tag;
            //                newMarker.ToolTipText = m.ToolTipText;
            //                break;
            //            }
            //        }
            //        newMarker.IsVisible = true;
            //        mouseOverMarkers.Markers.Add(newMarker);
            //        gmap1.Overlays.Add(mouseOverMarkers);

            //        gmap1.Zoom++;
            //        gmap1.Zoom--;
            //        markerOverOnce = true;
            //    }
            //}
            //catch(Exception)
            //{

            //}                
        }

        //see above function for details about why this does not work

        //highlight moueover marker and corresponding pickup/dropoff marker
        //[MethodImpl(MethodImplOptions.Synchronized)]
        private void gmap1_OnMarkerLeave(GMapMarker item)
        {
            //try
            //{
            //    markerOverOnce = false;
            //    mouseOverMarkers.Markers.Clear();
            //    gmap1.Overlays.Remove(mouseOverMarkers);
            //    gmap1.Zoom++;
            //    gmap1.Zoom--;
            //}
            //catch (Exception)
            //{
               
            //}
        }

        //function that allows me to manually draw the listbox and its items - allows for the red and green text
        private void lb_Destinations_DrawItem(object sender, DrawItemEventArgs e)
        {
            e.DrawBackground();
            Graphics g = e.Graphics;
            g.FillRectangle(new SolidBrush(Color.White), e.Bounds);
            
            if (lb_Destinations.Items[e.Index].ToString().Contains("Pickup"))           
                g.DrawString(lb_Destinations.Items[e.Index].ToString(), e.Font, new SolidBrush(Color.Green), new PointF(e.Bounds.X, e.Bounds.Y));                
            
            else
                g.DrawString(lb_Destinations.Items[e.Index].ToString(), e.Font, new SolidBrush(Color.Red), new PointF(e.Bounds.X, e.Bounds.Y));

            // Selected. Draw with the system highlight color.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
            {
                g.FillRectangle(new SolidBrush(SystemColors.MenuHighlight), e.Bounds);                
                e.Graphics.DrawString(lb_Destinations.Items[e.Index].ToString(), this.Font,
                    SystemBrushes.HighlightText, e.Bounds.Left,
                        e.Bounds.Top);
            }
            e.DrawFocusRectangle();
        }

        //used to fix an issue with the message box displaying multiple times if a dropoff marker is clicked and
        private void gmap1_MouseMove(object sender, MouseEventArgs e)
        {
            MBShown = false;
        }
    }
}
